/*
	Licensed to the Apache Software Foundation (ASF) under one
	or more contributor license agreements.  See the NOTICE file
	distributed with this work for additional information
	regarding copyright ownership.  The ASF licenses this file
	to you under the Apache License, Version 2.0 (the
	"License"); you may not use this file except in compliance
	with the License.  You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

	Unless required by applicable law or agreed to in writing,
	software distributed under the License is distributed on an
	"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
	KIND, either express or implied.  See the License for the
	specific language governing permissions and limitations
	under the License.
*/

/**
 * @file ecp2_FP256BN.h
 * @author Mike Scott
 * @brief ECP2 Header File
 *
 */

#ifndef ECP2_FP256BN_H
#define ECP2_FP256BN_H

#include "fp2_FP256BN.h"
#include "config_curve_FP256BN.h"

/**
	@brief ECP2 Structure - Elliptic Curve Point over quadratic extension field
*/

typedef struct
{
    int inf; /**< Infinity Flag */
    FP2_FP256BN x;   /**< x-coordinate of point */
    FP2_FP256BN y;   /**< y-coordinate of point */
    FP2_FP256BN z;   /**< z-coordinate of point */
} ECP2_FP256BN;


/* Curve Params - see rom_zzz.c */
extern const int CURVE_A_FP256BN;		/**< Elliptic curve A parameter */
extern const int CURVE_B_I_FP256BN;		/**< Elliptic curve B parameter */
extern const BIG_256_28 CURVE_B_FP256BN;     /**< Elliptic curve B parameter */
extern const BIG_256_28 CURVE_Order_FP256BN; /**< Elliptic curve group order */
extern const BIG_256_28 CURVE_Cof_FP256BN;   /**< Elliptic curve cofactor */
extern const BIG_256_28 CURVE_Bnx_FP256BN;   /**< Elliptic curve parameter */

extern const BIG_256_28 Fra_FP256BN; /**< real part of BN curve Frobenius Constant */
extern const BIG_256_28 Frb_FP256BN; /**< imaginary part of BN curve Frobenius Constant */


/* Generator point on G1 */
extern const BIG_256_28 CURVE_Gx_FP256BN; /**< x-coordinate of generator point in group G1  */
extern const BIG_256_28 CURVE_Gy_FP256BN; /**< y-coordinate of generator point in group G1  */

/* For Pairings only */

/* Generator point on G2 */
extern const BIG_256_28 CURVE_Pxa_FP256BN; /**< real part of x-coordinate of generator point in group G2 */
extern const BIG_256_28 CURVE_Pxb_FP256BN; /**< imaginary part of x-coordinate of generator point in group G2 */
extern const BIG_256_28 CURVE_Pya_FP256BN; /**< real part of y-coordinate of generator point in group G2 */
extern const BIG_256_28 CURVE_Pyb_FP256BN; /**< imaginary part of y-coordinate of generator point in group G2 */

/* ECP2 E(Fp2) prototypes */
/**	@brief Tests for ECP2 point equal to infinity
 *
	@param P ECP2 point to be tested
	@return 1 if infinity, else returns 0
 */
extern int ECP2_FP256BN_isinf(ECP2_FP256BN *P);
/**	@brief Copy ECP2 point to another ECP2 point
 *
	@param P ECP2 instance, on exit = Q
	@param Q ECP2 instance to be copied
 */
extern void ECP2_FP256BN_copy(ECP2_FP256BN *P,ECP2_FP256BN *Q);
/**	@brief Set ECP2 to point-at-infinity
 *
	@param P ECP2 instance to be set to infinity
 */
extern void ECP2_FP256BN_inf(ECP2_FP256BN *P);
/**	@brief Tests for equality of two ECP2s
 *
	@param P ECP2 instance to be compared
	@param Q ECP2 instance to be compared
	@return 1 if P=Q, else returns 0
 */
extern int ECP2_FP256BN_equals(ECP2_FP256BN *P,ECP2_FP256BN *Q);
/**	@brief Converts an ECP2 point from Projective (x,y,z) coordinates to affine (x,y) coordinates
 *
	@param P ECP2 instance to be converted to affine form
 */
extern void ECP2_FP256BN_affine(ECP2_FP256BN *P);
/**	@brief Extract x and y coordinates of an ECP2 point P
 *
	If x=y, returns only x
	@param x FP2 on exit = x coordinate of point
	@param y FP2 on exit = y coordinate of point (unless x=y)
	@param P ECP2 instance (x,y)
	@return -1 if P is point-at-infinity, else 0
 */
extern int ECP2_FP256BN_get(FP2_FP256BN *x,FP2_FP256BN *y,ECP2_FP256BN *P);
/**	@brief Formats and outputs an ECP2 point to the console, converted to affine coordinates
 *
	@param P ECP2 instance to be printed
 */
extern void ECP2_FP256BN_output(ECP2_FP256BN *P);
/**	@brief Formats and outputs an ECP2 point to the console, in projective coordinates
 *
	@param P ECP2 instance to be printed
 */
extern void ECP2_FP256BN_outputxyz(ECP2_FP256BN *P);
/**	@brief Formats and outputs an ECP2 point to an octet string
 *
	The octet string is created in the form x|y.
	Convert the real and imaginary parts of the x and y coordinates to big-endian base 256 form.
	@param S output octet string
	@param P ECP2 instance to be converted to an octet string
 */
extern void ECP2_FP256BN_toOctet(octet *S,ECP2_FP256BN *P);
/**	@brief Creates an ECP2 point from an octet string
 *
	The octet string is in the form x|y
	The real and imaginary parts of the x and y coordinates are in big-endian base 256 form.
	@param P ECP2 instance to be created from the octet string
	@param S input octet string
	return 1 if octet string corresponds to a point on the curve, else 0
 */
extern int ECP2_FP256BN_fromOctet(ECP2_FP256BN *P,octet *S);
/**	@brief Calculate Right Hand Side of curve equation y^2=f(x)
 *
	Function f(x)=x^3+Ax+B
	Used internally.
	@param r FP2 value of f(x)
	@param x FP2 instance
 */
extern void ECP2_FP256BN_rhs(FP2_FP256BN *r,FP2_FP256BN *x);
/**	@brief Set ECP2 to point(x,y) given x and y
 *
	Point P set to infinity if no such point on the curve.
	@param P ECP2 instance to be set (x,y)
	@param x FP2 x coordinate of point
	@param y FP2 y coordinate of point
	@return 1 if point exists, else 0
 */
extern int ECP2_FP256BN_set(ECP2_FP256BN *P,FP2_FP256BN *x,FP2_FP256BN *y);
/**	@brief Set ECP to point(x,[y]) given x
 *
	Point P set to infinity if no such point on the curve. Otherwise y coordinate is calculated from x.
	@param P ECP instance to be set (x,[y])
	@param x BIG x coordinate of point
	@return 1 if point exists, else 0
 */
extern int ECP2_FP256BN_setx(ECP2_FP256BN *P,FP2_FP256BN *x);
/**	@brief Negation of an ECP2 point
 *
	@param P ECP2 instance, on exit = -P
 */
extern void ECP2_FP256BN_neg(ECP2_FP256BN *P);
/**	@brief Doubles an ECP2 instance P
 *
	@param P ECP2 instance, on exit =2*P
 */
extern int ECP2_FP256BN_dbl(ECP2_FP256BN *P);
/**	@brief Adds ECP2 instance Q to ECP2 instance P
 *
	@param P ECP2 instance, on exit =P+Q
	@param Q ECP2 instance to be added to P
 */
extern int ECP2_FP256BN_add(ECP2_FP256BN *P,ECP2_FP256BN *Q);
/**	@brief Subtracts ECP instance Q from ECP2 instance P
 *
	@param P ECP2 instance, on exit =P-Q
	@param Q ECP2 instance to be subtracted from P
 */
extern void ECP2_FP256BN_sub(ECP2_FP256BN *P,ECP2_FP256BN *Q);
/**	@brief Multiplies an ECP2 instance P by a BIG, side-channel resistant
 *
	Uses fixed sized windows.
	@param P ECP2 instance, on exit =b*P
	@param b BIG number multiplier

 */
extern void ECP2_FP256BN_mul(ECP2_FP256BN *P,BIG_256_28 b);
/**	@brief Multiplies an ECP2 instance P by the internal modulus p, using precalculated Frobenius constant f
 *
	Fast point multiplication using Frobenius
	@param P ECP2 instance, on exit = p*P
	@param f FP2 precalculated Frobenius constant

 */
extern void ECP2_FP256BN_frob(ECP2_FP256BN *P,FP2_FP256BN *f);
/**	@brief Calculates P=b[0]*Q[0]+b[1]*Q[1]+b[2]*Q[2]+b[3]*Q[3]
 *
	@param P ECP2 instance, on exit = b[0]*Q[0]+b[1]*Q[1]+b[2]*Q[2]+b[3]*Q[3]
	@param Q ECP2 array of 4 points
	@param b BIG array of 4 multipliers
 */
extern void ECP2_FP256BN_mul4(ECP2_FP256BN *P,ECP2_FP256BN *Q,BIG_256_28 *b);

/**	@brief Maps random BIG to curve point of correct order
 *
	@param P ECP2 instance of correct order
	@param w OCTET byte array to be mapped
 */
extern void ECP2_FP256BN_mapit(ECP2_FP256BN *P,octet *w);

#endif
