/*
	Licensed to the Apache Software Foundation (ASF) under one
	or more contributor license agreements.  See the NOTICE file
	distributed with this work for additional information
	regarding copyright ownership.  The ASF licenses this file
	to you under the Apache License, Version 2.0 (the
	"License"); you may not use this file except in compliance
	with the License.  You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

	Unless required by applicable law or agreed to in writing,
	software distributed under the License is distributed on an
	"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
	KIND, either express or implied.  See the License for the
	specific language governing permissions and limitations
	under the License.
*/

/**
 * @file fp_NIST521.h
 * @author Mike Scott
 * @brief FP Header File
 *
 */

#ifndef FP_NIST521_H
#define FP_NIST521_H

#include "big_528_28.h"
#include "config_field_NIST521.h"


/**
	@brief FP Structure - quadratic extension field
*/

typedef struct
{
    BIG_528_28 g;	/**< Big representation of field element */
    sign32 XES;	/**< Excess */
} FP_NIST521;


/* Field Params - see rom.c */
extern const BIG_528_28 Modulus_NIST521;	/**< Actual Modulus set in romf_yyy.c */
extern const BIG_528_28 R2modp_NIST521;	/**< Montgomery constant */
extern const chunk MConst_NIST521;		/**< Constant associated with Modulus - for Montgomery = 1/p mod 2^BASEBITS */


#define MODBITS_NIST521 MBITS_NIST521                        /**< Number of bits in Modulus for selected curve */
#define TBITS_NIST521 (MBITS_NIST521%BASEBITS_528_28)           /**< Number of active bits in top word */
#define TMASK_NIST521 (((chunk)1<<TBITS_NIST521)-1)          /**< Mask for active bits in top word */
#define FEXCESS_NIST521 ((sign32)1<<MAXXES_NIST521)	     /**< 2^(BASEBITS*NLEN-MODBITS) - normalised BIG can be multiplied by more than this before reduction */
#define OMASK_NIST521 (-((chunk)(1)<<TBITS_NIST521))         /**<  for masking out overflow bits */

//#define FUSED_MODMUL
//#define DEBUG_REDUCE

/* FP prototypes */

/**	@brief Tests for FP equal to zero mod Modulus
 *
	@param x BIG number to be tested
	@return 1 if zero, else returns 0
 */
extern int FP_NIST521_iszilch(FP_NIST521 *x);


/**	@brief Set FP to zero
 *
	@param x FP number to be set to 0
 */
extern void FP_NIST521_zero(FP_NIST521 *x);

/**	@brief Copy an FP
 *
	@param y FP number to be copied to
	@param x FP to be copied from
 */
extern void FP_NIST521_copy(FP_NIST521 *y,FP_NIST521 *x);

/**	@brief Copy from ROM to an FP
 *
	@param y FP number to be copied to
	@param x BIG to be copied from ROM
 */
extern void FP_NIST521_rcopy(FP_NIST521 *y,const BIG_528_28 x);


/**	@brief Compares two FPs
 *
	@param x FP number
	@param y FP number
	@return 1 if equal, else returns 0
 */
extern int FP_NIST521_equals(FP_NIST521 *x,FP_NIST521 *y);


/**	@brief Conditional constant time swap of two FP numbers
 *
	Conditionally swaps parameters in constant time (without branching)
	@param x an FP number
	@param y another FP number
	@param s swap takes place if not equal to 0
 */
extern void FP_NIST521_cswap(FP_NIST521 *x,FP_NIST521 *y,int s);
/**	@brief Conditional copy of FP number
 *
	Conditionally copies second parameter to the first (without branching)
	@param x an FP number
	@param y another FP number
	@param s copy takes place if not equal to 0
 */
extern void FP_NIST521_cmove(FP_NIST521 *x,FP_NIST521 *y,int s);
/**	@brief Converts from BIG integer to residue form mod Modulus
 *
	@param x BIG number to be converted
	@param y FP result
 */
extern void FP_NIST521_nres(FP_NIST521 *y,BIG_528_28 x);
/**	@brief Converts from residue form back to BIG integer form
 *
	@param y FP number to be converted to BIG
	@param x BIG result
 */
extern void FP_NIST521_redc(BIG_528_28 x,FP_NIST521 *y);
/**	@brief Sets FP to representation of unity in residue form
 *
	@param x FP number to be set equal to unity.
 */
extern void FP_NIST521_one(FP_NIST521 *x);
/**	@brief Reduces DBIG to BIG exploiting special form of the modulus
 *
	This function comes in different flavours depending on the form of Modulus that is currently in use.
	@param r BIG number, on exit = d mod Modulus
	@param d DBIG number to be reduced
 */
extern void FP_NIST521_mod(BIG_528_28 r,DBIG_528_28 d);

#ifdef FUSED_MODMUL
extern void FP_NIST521_modmul(BIG_528_28,BIG_528_28,BIG_528_28);
#endif

/**	@brief Fast Modular multiplication of two FPs, mod Modulus
 *
	Uses appropriate fast modular reduction method
	@param x FP number, on exit the modular product = y*z mod Modulus
	@param y FP number, the multiplicand
	@param z FP number, the multiplier
 */
extern void FP_NIST521_mul(FP_NIST521 *x,FP_NIST521 *y,FP_NIST521 *z);
/**	@brief Fast Modular multiplication of an FP, by a small integer, mod Modulus
 *
	@param x FP number, on exit the modular product = y*i mod Modulus
	@param y FP number, the multiplicand
	@param i a small number, the multiplier
 */
extern void FP_NIST521_imul(FP_NIST521 *x,FP_NIST521 *y,int i);
/**	@brief Fast Modular squaring of an FP, mod Modulus
 *
	Uses appropriate fast modular reduction method
	@param x FP number, on exit the modular product = y^2 mod Modulus
	@param y FP number, the number to be squared

 */
extern void FP_NIST521_sqr(FP_NIST521 *x,FP_NIST521 *y);
/**	@brief Modular addition of two FPs, mod Modulus
 *
	@param x FP number, on exit the modular sum = y+z mod Modulus
	@param y FP number
	@param z FP number
 */
extern void FP_NIST521_add(FP_NIST521 *x,FP_NIST521 *y,FP_NIST521 *z);
/**	@brief Modular subtraction of two FPs, mod Modulus
 *
	@param x FP number, on exit the modular difference = y-z mod Modulus
	@param y FP number
	@param z FP number
 */
extern void FP_NIST521_sub(FP_NIST521 *x,FP_NIST521 *y,FP_NIST521 *z);
/**	@brief Modular division by 2 of an FP, mod Modulus
 *
	@param x FP number, on exit =y/2 mod Modulus
	@param y FP number
 */
extern void FP_NIST521_div2(FP_NIST521 *x,FP_NIST521 *y);
/**	@brief Fast Modular exponentiation of an FP, to the power of a BIG, mod Modulus
 *
	@param x FP number, on exit  = y^z mod Modulus
	@param y FP number
	@param z BIG number exponent
 */
extern void FP_NIST521_pow(FP_NIST521 *x,FP_NIST521 *y,BIG_528_28 z);
/**	@brief Fast Modular square root of a an FP, mod Modulus
 *
	@param x FP number, on exit  = sqrt(y) mod Modulus
	@param y FP number, the number whose square root is calculated

 */
extern void FP_NIST521_sqrt(FP_NIST521 *x,FP_NIST521 *y);
/**	@brief Modular negation of a an FP, mod Modulus
 *
	@param x FP number, on exit = -y mod Modulus
	@param y FP number
 */
extern void FP_NIST521_neg(FP_NIST521 *x,FP_NIST521 *y);
/**	@brief Outputs an FP number to the console
 *
	Converts from residue form before output
	@param x an FP number
 */
extern void FP_NIST521_output(FP_NIST521 *x);
/**	@brief Outputs an FP number to the console, in raw form
 *
	@param x a BIG number
 */
extern void FP_NIST521_rawoutput(FP_NIST521 *x);
/**	@brief Reduces possibly unreduced FP mod Modulus
 *
	@param x FP number, on exit reduced mod Modulus
 */
extern void FP_NIST521_reduce(FP_NIST521 *x);
/**	@brief normalizes FP
 *
	@param x FP number, on exit normalized
 */
extern void FP_NIST521_norm(FP_NIST521 *x);
/**	@brief Tests for FP a quadratic residue mod Modulus
 *
	@param x FP number to be tested
	@return 1 if quadratic residue, else returns 0 if quadratic non-residue
 */
extern int FP_NIST521_qr(FP_NIST521 *x);
/**	@brief Modular inverse of a an FP, mod Modulus
 *
	@param x FP number, on exit = 1/y mod Modulus
	@param y FP number
 */
extern void FP_NIST521_inv(FP_NIST521 *x,FP_NIST521 *y);




#endif
